unit IOIface;

interface

uses classes, adport, scktcomp, sysutils, comdialog, controls;

type

  TIOIfaceReceiveEvent = procedure( const Str : String ) of Object;
  TIOIfaceNotifyEvent = procedure( Sender: TObject ) of Object;

  // Abstract base class for a IO Interface
  TIOIface = Class( TComponent )
  private
    FOnReceive : TIOIfaceReceiveEvent;
    FOnConnect : TIOIfaceNotifyEvent;
    FOnDisconnect : TIOIfaceNotifyEvent;

  protected
    function GetConnected : Boolean; virtual; abstract;

  public
    procedure Connect; virtual; abstract;
    procedure Disconnect; virtual; abstract;
    procedure WriteString( const Str : String ); virtual; abstract;

    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;

    property Connected : Boolean read GetConnected;
    property OnReceive : TIOIfaceReceiveEvent read FOnReceive write FOnReceive;
    property OnConnect : TIOIfaceNotifyEvent read FOnConnect write FOnConnect;
    property OnDisconnect : TIOIfaceNotifyEvent read FOnDisconnect write FOnDisconnect;

    // Get the port as a displayable string eg 172.17.1.1:2222 or COM23
    function GetDisplayablePort : String; virtual; abstract;
  end;

  // Null IO Interface ( Can write anything, read nothing )
  TIOIfaceNull = Class( TIOIface )
  private

  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;

    procedure Connect; override;
    procedure Disconnect; override;
    procedure WriteString( const Str : String ); override;
    function GetConnected : Boolean; override;

    // Get the port as a displayable string ""
    function GetDisplayablePort : String; override;
  end;


  // IO Interface using a TCP socket
  TIOIfaceSocket = Class( TIOIface )
  private
    FSocket : TClientSocket;

    procedure SocketReceiveEventHandler( Sender: TObject; Socket: TCustomWinSocket );
    procedure SocketConnectEventHandler( Sender: TObject; Socket: TCustomWinSocket );
    procedure SocketDisconnectEventHandler( Sender: TObject; Socket: TCustomWinSocket );

    function GetHost : String;
    procedure SetHost( Host : String );
    function GetAddress : String;
    procedure SetAddress( Address : String );
    function GetPort : Integer;
    procedure SetPort( Port : Integer );

  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;

    procedure Connect; override;
    procedure Disconnect; override;
    procedure WriteString( const Str : String ); override;
    function GetConnected : Boolean; override;

    // Get the port as a displayable string "ip:port"
    function GetDisplayablePort : String; override;

    property Host : String read GetHost write SetHost;
    property Address : String read GetAddress write SetAddress;
    property Port : Integer read GetPort write SetPort;
  end;


  // IO Interface using a serial port
  TIOIfaceSerial = Class( TIOIface )
  private
    FComPort : TApdComPort;
    FUseRinlink : Boolean;

    procedure SerialReceiveEventHandler( CP : TObject; Count : Word );

    function GetComPort : String;
    procedure SetComPort( Port : String );
    function GetBaudRate : Integer;
    procedure SetBaudRate( BaudRate : Integer );
    function GetStopBits : Integer;
    procedure SetStopBits( StopBits : Integer );
    function GetDataBits : Integer;
    procedure SetDataBits( DataBits : Integer );
    function GetParityBits : TParity;
    procedure SetParityBits( Parity : TParity );
    procedure SetUseRinlink( UseRinLink : Boolean );

  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;

    procedure Connect; override;
    procedure Disconnect; override;
    procedure WriteString( const Str : String ); override;
    function GetConnected : Boolean; override;

    // Get the port as a displayable string "COMx"
    function GetDisplayablePort : String; override;

    property ComPort : String read GetComPort write SetComPort;
    property BaudRate : Integer read GetBaudRate write SetBaudRate;
    property StopBits : Integer read GetStopBits write SetStopBits;
    property DataBits : Integer read GetDataBits write SetDataBits;
    property ParityBits : TParity read GetParityBits write SetParityBits;
    property UseRinLink : Boolean read FUseRinLink write SetUseRinlink;
  end;


implementation


// TIOIface

constructor TIOIface.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FOnReceive := nil;
  FOnConnect := nil;
  FOnDisconnect := nil;
end;


destructor TIOIface.Destroy;
begin
  inherited Destroy;
end;


// TIOIfaceNull


constructor TIOIfaceNull.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
end;


destructor TIOIfaceNull.Destroy;
begin
  inherited Destroy;
end;


procedure TIOIfaceNull.Connect;
begin
  // Do nothing, except call the connect handler
  if ( Assigned( FOnConnect ) ) then
    FOnConnect( Self );
end;


procedure TIOIfaceNull.Disconnect;
begin
  // Do nothing, except call the disconnect handler
  if ( Assigned( FOnDisconnect ) ) then
    FOnDisconnect( Self );
end;


procedure TIOIfaceNull.WriteString( const Str : String );
begin
 // Do nothing
end;


function TIOIfaceNull.GetConnected : Boolean;
begin
  Result := False;
end;


function TIOIfaceNull.GetDisplayablePort : String;
begin
  Result := '';
end;


// TIOIfaceSocket


constructor TIOIfaceSocket.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FSocket := TClientSocket.Create( nil );
  FSocket.OnRead := SocketReceiveEventHandler;
  FSocket.OnConnect := SocketConnectEventHandler;
  FSocket.OnDisconnect := SocketDisconnectEventHandler;
end;


destructor TIOIfaceSocket.Destroy;
begin
  FSocket.Destroy();
  inherited Destroy;
end;


procedure TIOIfaceSocket.SocketReceiveEventHandler( Sender: TObject; Socket: TCustomWinSocket );
var S : String;
begin
  S := Socket.ReceiveText;
  if Assigned( FOnReceive ) then
    FOnReceive( S );
end;


procedure TIOIfaceSocket.SocketConnectEventHandler( Sender: TObject; Socket: TCustomWinSocket );
begin
  if Assigned( FOnConnect ) then
    FOnConnect( Self );
end;


procedure TIOIfaceSocket.SocketDisconnectEventHandler( Sender: TObject; Socket: TCustomWinSocket );
begin
  if Assigned( FOnDisconnect ) then
    FOnDisconnect( Self );
end;


procedure TIOIfaceSocket.Connect;
begin
  FSocket.Open();
end;


procedure TIOIfaceSocket.Disconnect;
begin
  FSocket.Close();
end;


procedure TIOIfaceSocket.WriteString( const Str : String );
begin
  FSocket.Socket.SendText( Str );
end;


function TIOIfaceSocket.GetConnected : Boolean;
begin
  Result := FSocket.Active; 
end;


function TIOIfaceSocket.GetDisplayablePort : String;
var host : String;
begin
  if (FSocket.Host <> '') then
    host := FSocket.Host
  else
    host := FSocket.Address;
  Result := Format('%s:%d', [host, FSocket.Port]);
end;


function TIOIfaceSocket.GetHost : String;
begin
  Result := FSocket.Host;
end;


procedure TIOIfaceSocket.SetHost( Host : String );
begin
  FSocket.Host := Host;
end;


function TIOIfaceSocket.GetAddress : String;
begin
  Result := FSocket.Address;
end;


procedure TIOIfaceSocket.SetAddress( Address : String );
begin
  FSocket.Address := Address;
end;


function TIOIfaceSocket.GetPort : Integer;
begin
  Result := FSocket.Port;
end;


procedure TIOIfaceSocket.SetPort( Port : Integer );
begin
  FSocket.Port := Port;
end;


// TIOIfaceSerial

constructor TIOIfaceSerial.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FComPort := TApdComPort.Create( nil );
  FComPort.OnTriggerAvail := SerialReceiveEventHandler;
  FUseRinLink := true;
end;


destructor TIOIfaceSerial.Destroy;
begin
  FComPort.Destroy();
  inherited Destroy;
end;


procedure TIOIfaceSerial.SerialReceiveEventHandler( CP : TObject; Count : Word );
var
  I : Word;
  S : String;
begin
  S := '';
  for I := 1 to Count do
  begin
    S := S + FComPort.GetChar();
  end;
  if Assigned( FOnReceive ) then
    FOnReceive( S );
end;


procedure TIOIfaceSerial.Connect;
begin
  FComPort.Open := true;
  
  // Call the connect handler
  if ( Assigned( FOnConnect ) ) then
    FOnConnect( Self );
end;


procedure TIOIfaceSerial.Disconnect;
begin
  FComPort.Open := false;
  // Call the disconnect handler
  if ( Assigned( FOnDisconnect ) ) then
    FOnDisconnect( Self );
end;


procedure TIOIfaceSerial.WriteString( const Str : String );
begin
  FComPort.PutString( Str );
end;


function TIOIfaceSerial.GetConnected : Boolean;
begin
  Result := FComPort.Open;
end;


function TIOIfaceSerial.GetDisplayablePort : String;
begin
  Result := Self.GetComPort();
end;


function TIOIfaceSerial.GetComPort : String;
begin
  Result := Format( 'COM%d', [ FComPort.ComNumber ] );
end;


function port_no(port_str: string): integer;
var
  i, p : integer;
begin
  p := 0;
  for i := 4 to length(port_str) do
    p := (p * 10) + (ord(port_str[i]) - ord('0'));
  if p = 0 then
    p := 1;
  result := p;
end;


procedure TIOIfaceSerial.SetComPort( Port : String );
begin
  FComPort.ComNumber := port_no( Port );
end;


function TIOIfaceSerial.GetBaudRate : Integer;
begin
  Result := FComPort.Baud;
end;


procedure TIOIfaceSerial.SetBaudRate( BaudRate : Integer );
begin
  FComPort.Baud := BaudRate;
end;


function TIOIfaceSerial.GetStopBits : Integer;
begin
  Result := FComPort.StopBits;
end;


procedure TIOIfaceSerial.SetStopBits( StopBits : Integer );
begin
  FComPort.StopBits := StopBits;
end;


function TIOIfaceSerial.GetDataBits : Integer;
begin
  Result := FComPort.DataBits;
end;


procedure TIOIfaceSerial.SetDataBits( DataBits : Integer );
begin
  FComPort.DataBits := DataBits;
end;


function TIOIfaceSerial.GetParityBits : TParity;
begin
  Result := FComPort.Parity;
end;


procedure TIOIfaceSerial.SetParityBits( Parity : TParity );
begin
  FComPort.Parity := Parity;
// for testing with comms splitter
  FComPort.RTS := false;
  FComPort.DTR := true;
end;


procedure TIOIfaceSerial.SetUseRinlink( UseRinLink : Boolean );
begin
//FUseRinlink := UseRinLink;
//if (UseRinLink) then
//begin
    FComPort.RTS := false;
    FComPort.DTR := true;
//end
//else
//begin
//  FComPort.RTS := false;
//  FComPort.DTR := false;
//end;
end;
end.

