unit kenButton2;

// Title : Kens LCD Buttons
// Author : Ken Bray
// Date : 27 May 2003
//
// Description:
//   This library handles mouse clicks on a TPaintBox.  There are 2 aspects to
//   this: visible feedback and events for programs to use.
//
//   Visible feedback is provided by the and'ing of the button with another
//   bitmap.  3 bitmaps are provided for this but custom bitmaps may be added.
//   The 3 provided bitmaps are A) no feedback, B) eliptical button and C)
//   rectangular button.  Note: no feedback is provided for any set interval.

//   Three events are provided for the viewer package to use: 1) OnMouseDown, 2)
//   OnMouseUp and 3) OnMouseClick.
//
//   All buttons are logically rectangular in shape.  The size and position of
//   the buttons must be described before use.  Mouse clicks which fall outside
//   of the defined regions are ignored.
//
//   An example program is available called R300LCDTest.
//
// Limitations:
//   1. Only a single button can be pressed at any time
//   2. An image resize while the button is held down will cause incorrect
//     operation (I could not find a way to do this anyway)
//   3. Maximum number of buttons is 50.  This can be changed by changing the
//     MaxNumButtons constant.
//
// Usage:
//   1. Define all buttons using the buttons variable.  Left, right, top and
//     bottom are all defined assuming an image of 1000 x 1000.  The maskIndex
//     defines the shape of visible response to a click.
//   2. Define the number of buttons using the numButtons variable.
//   3. Define the TPaintBox to operate on using the image property.  Note: The
//     OnMouseDown, OnMouseUp and OnMouseMove events of the TPaintBox will be
//     used
//   4. Assign handlers to the events (OnMouseClick, OnMouseDown and OnMouseUp
//     events
//
// Other notes:
//   1. The button index given in the events is the index in the button
//     definition array.
//   2. Button definitions do not have to be in a particular order.  They can
//     also overlap but the first button found containing the mouse pointer
//     will be used. 

interface

uses windows, classes, controls, graphics, dialogs, sysutils, extctrls,
  kenbitmap2;

const
  MaxNumButtons = 50;   // Maximum number of buttons which can be used
  MaskColor = clSilver; // Color used to and with pressed buttons

type
  TKenLCDMouseClickEvent = procedure (Sender: TObject; btnIndex : integer; msDown : integer) of object;
  TKenLCDMouseDownEvent = procedure (Sender: TObject; btnIndex : integer) of object;
  TKenLCDMouseUpEvent = procedure (Sender: TObject; btnIndex : integer) of object;

  TKenLCDButtons = class(TObject)
  private
    // storage for user events
    fMouseClick : TKenLCDMouseClickEvent;
    fMouseUp : TKenLCDMouseUpEvent;
    fMouseDown : TKenLCDMouseDownEvent;

    // items used for saving and restoring button state
    fStart : TDateTime;   // time of mousedown
    fBtnDown : integer;   // value is -1 or index of button currently down

    // handlers for paintbox mouse events
    procedure MouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
  public
    kbm : TKenBitmap;  // resized version of key file
    image : TImage;

    ready : boolean;


    constructor create;
    destructor destroy; override;

    procedure Setup(filename:string; aImage:TImage);

    procedure MonitorTimeout(TimeoutMs : integer);

  published
    property OnMouseClick : TKenLCDMouseClickEvent read fMouseClick write fMouseClick;
    property OnMouseUp : TKenLCDMouseUpEvent read fMouseUp write fMouseUp;
    property OnMouseDown : TKenLCDMouseDownEvent read fMouseDown write fMouseDown;
  end;

implementation

uses kenmimicdisp;

constructor TKenLCDButtons.create;
begin
  inherited create;

  kbm := TKenBitmap.create;
  fBtnDown := -1;
end;

destructor TKenLCDButtons.destroy;
begin
  kbm.free;

  inherited destroy;
end;

procedure TKenLCDButtons.Setup(filename:string; aImage:TImage);
var
  kbmTemp : TKenBitmap;
begin
  ready := FALSE;
  fBtnDown := -1;

  if filename <> '' then begin
    image := aImage;
    aImage.OnMouseDown := MouseDown;
    aImage.OnMouseMove := MouseMove;
    aImage.OnMouseUp := MouseUp;

    kbmtemp := TKenBitmap.create;
    try
      kbmTemp.LoadFromFile(filename);
      kbm.CopyStretch(kbmTemp, image.width, image.height);
    finally
      kbmtemp.free;
    end;

    ready := TRUE;
  end;
end;

procedure TKenLCDButtons.MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  // never required
  // todo:help for buttons?
end;

procedure TKenLCDButtons.MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i,j : integer;
  k : integer; //key id
begin
  if not ready then exit;

  k := kbm.GetData(X,Y);

  // restore previous button image
  if fBtnDown >= 0 then begin
    MouseUp(sender,button,Shift,X,Y);
  end;

  if k < kbm.numColours then begin
    // store button ID
    fBtnDown := k;

    // start mouse down timer
    fStart := Now;

    // do button image
    for j := 0 to kbm.Height-1 do begin
      for i := 0 to kbm.Width-1 do begin
        if (kbm.data[i,j] = fBtnDown) then
          image.Canvas.Pixels[i,j] := image.Canvas.Pixels[i,j] xor $00444444;
      end;
    end;

    // call user event
    if assigned(Onmousedown) then OnMouseDown(self,fBtnDown);
  end;
end;

procedure TKenLCDButtons.MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i,j : integer;
begin
  if not ready then exit;

  if fBtnDown >= 0 then begin
    // restore button image
    for j := 0 to kbm.Height-1 do begin
      for i := 0 to kbm.Width-1 do begin
        if (kbm.data[i,j] = fBtnDown) then
          image.Canvas.Pixels[i,j] := image.Canvas.Pixels[i,j] xor $00444444;
      end;
    end;

    // reset state
    j := fBtnDown;
    fBtnDown := -1;

    // call client functions
    if assigned(Onmouseup) then OnMouseUp(self,j);
    if Assigned(onmouseclick) then OnMouseClick(self,j,Round((Now-fstart)*86400000));
  end;
end;

procedure TKenLCDButtons.MonitorTimeout(TimeoutMs : integer);
begin
  if ((Now-fStart)*86400000) >= timeoutMs then
    MouseUp(image,mbLeft,[ssLeft],0,0);
end;

end.
