unit kenlcd2;

// Title : Kens LCD
// Author : Ken Bray
// Date : 15 Dec 2003
// version : 1.2
//
// Description:
//   This library handles the drawing of a custom LCD onto a Timage.
//
//   Segments are defined using KensLCDTool (TEST-526).
//
//   The segOn variable is used to turn segments on and off.  This variable is
//   an array of booleans.  This can be used as a linear array or matrix
//   (intended to match the LCD specification).  SetSegsOnOff can be used to
//   set segOn as a group.  It takes a hex string as a parameter.
//
//   Use the Draw methods to draw the LCD.  DrawFast will draw the LCD with a
//   blank background.   DrawWithBackground supports an image background.
//
//   The WE2110 viewer can be used as an example program (as can TEST-526).
//
//   This lcd component is used with TKenMimicDisp (kenmimicdisp.pas) to provide
//   a complete mimic solution.
//
// Limitations:
//
// Usage:
//   1. Create and define the LCD off color and on color.
//   2. Use SetupLCDSize to define the width, height and lcd filename.
//   3. If a background image is used, call SetupLCDBackground.
//   4. Call Drawfast or DrawWithBackground to draw the LCD segments onto a
//      client canvas.
//
// Other notes:
//
//

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, extctrls,
     forms, dialogs, kenbitmap2;

const
  PalFlag = 0;               // PC_NOCOLLAPSE or PC_RESERVED;
  PalOn = 101;               // palette number for on color
  PalOff = 100;              // palette number for off color

type

  TKenLCD = class (TObject)
  private
    colorMap : array of byte;
    hPal : hPalette;   // pallette for fbmOut
    bmBG1 : Tbitmap;  // bitmap used for background images
    bmBG2 : Tbitmap;  // bitmap used for background images

    procedure SetupImage(w, h : integer);
    procedure SetASegment(segnum : Integer; onoff : Boolean);
  public
    // stuff that you probably want to let the component sort out itself
    kbm : TKenBitmap;  // resized version of LCD file
    fbmOut : Tbitmap;  // correctly drawn LCD ready to be copied to display
    height, width : integer;

    // setup things you will want to write
    onColor, offColor : TColor;

    // used by the user to turn segments on and off
    segOn : array of boolean;

    // set to ready after LCD is setup
    ready : boolean;

    constructor create;
    destructor destroy; override;

    // init LCD
    procedure SetupLCDSize(lcdWidth, lCDHeight: integer; kmpFilename: string);  // call on start and when resized

    // Draw the LCD onto a canvas.  If width and height do not match previously
    //  stored values, it is resized first.
    procedure DrawFast(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
    procedure DrawOnOffFast(onOff:boolean; aleft, atop, awidth, aheight : integer; acanvas : TCanvas);


    // if the LCD must be drawn onto a variable background, use these instead.
    procedure SetupLCDBackground(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
    procedure DrawWithBackground(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
    procedure DrawOnOffWithBackground(onOff:boolean; aleft, atop, awidth, aheight : integer; acanvas : TCanvas);

    // read and write segOn variable as a hex text string
    procedure SetSegsOnOff(hex : string);
    function  GetSegsOnOff(numBytes : integer) : string;

    function GetNumSegments : Integer;

    property NumSegments : Integer read GetNumSegments;
  end;

implementation

constructor TKenLCD.create;
var i : integer;
begin
  inherited create;

  ready := FALSE;
  offColor := clNone;
  onColor := clYellow;

  // create bitmap to store final LCD with all segments correctly on/off
  fbmOut := TBitmap.Create;
  fbmOut.PixelFormat := pf8bit;
  fbmOut.Transparent := false;

  kbm := TKenBitmap.create;

  bmBG1 := TBitmap.create;
  bmBG1.PixelFormat := pf32bit;
  bmBG1.Transparent := false;
  bmBG2 := TBitmap.create;
  bmBG2.PixelFormat := pf32bit;
  bmBG2.Transparent := false;

  // default info
  height := 20;
  width := 40;

  SetLength(colorMap,500); //default
  SetLength(segOn,500);

  // initialise all segments to ON
  for i := 0 to 500-1 do begin
    segOn[i] := TRUE;
  end;
end;

destructor TKenLCD.destroy;
begin
  fbmOut.Free;
  kbm.free;
  bmBG1.Free;
  bmBG2.Free;

  // Free palette object created by SetupImage method
  if hPal <> 0 then DeleteObject(hPal);

  inherited destroy;
end;

procedure TKenLCD.SetupLCDSize(lcdWidth, lCDHeight: integer; kmpFilename: string);
var
  kbmTemp : TKenBitmap;
begin
  ready := FALSE;

  // generate on/off colour palette if not already done
  if hpal = 0 then begin
    SetupImage(width,height);
  end;

  width := lcdWidth;
  height := lCDHeight;
  fbmOut.Width := width;
  fbmOut.Height := height;

  kbmtemp := TKenBitmap.create;
  try
    kbmTemp.LoadFromFile(kmpFilename);
    kbm.CopyStretch(kbmTemp, width, height);
  finally
    kbmtemp.free;
  end;
  
  SetLength(colorMap,kbm.backgroundColour+1);
  SetLength(segOn,kbm.backgroundColour+1);

  ready := TRUE;
end;


procedure TKenLCD.DrawFast(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
var
  i, j : integer;
  lout : PByteArray;
begin
  // resize if required
  if ((hPal = 0) or (awidth <> fbmout.width) or
      (aheight <> fbmOut.height)) then SetupLCDSize(awidth,aheight,kbm.filename);

  // generate color map for each segment (on or off)
  for i := 0 to kbm.backgroundColour-1 do begin
    colorMap[i] := ord(segon[i])+PalOff;
  end;
  colorMap[kbm.backgroundColour] := PalOff;

  // use color map to copy pixels
  for j := 0 to kbm.Height-1 do begin
    lout := fbmOut.ScanLine[j];
    for i := 0 to kbm.Width-1 do begin
      lout[i] := colorMap[kbm.data[i,j]];
    end;
  end;

  // draw onto customer canvas
  aCanvas.CopyMode := cmSrcCopy;
  aCanvas.Draw(aleft,atop,fbmOut);
end;

procedure TKenLCD.DrawOnOffFast(onOff:boolean; aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
var
  i, j : integer;
  lout : PByteArray;
  c : Tcolor;
begin
  // resize if required
  if ((hPal = 0) or (awidth <> fbmout.width) or
      (aheight <> fbmOut.height)) then SetupLCDSize(awidth,aheight,kbm.filename);

  if onOff then c := onColor
  else c := offColor;

  // use color map to copy pixels
  for j := 0 to kbm.Height-1 do begin
    lout := fbmOut.ScanLine[j];
    for i := 0 to kbm.Width-1 do begin
      if (kbm.data[i,j] <> kbm.backgroundColour) then
        lout[i] := c;
    end;
  end;

  // draw onto customer canvas
  aCanvas.CopyMode := cmSrcCopy;
  aCanvas.Draw(aleft,atop,fbmOut);
end;

procedure TKenLCD.SetupLCDBackground(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
var
  srcRect, destRect : TRect;
begin
  bmBG1.Width := awidth;
  bmBG1.Height := aheight;
  bmBG2.Width := awidth;
  bmBG2.Height := aheight;

  srcRect := Rect(aleft,atop,aleft+awidth,atop+aheight);
  destRect := Rect(0,0,bmBG1.width,bmBG1.height);
  bmBG1.Canvas.CopyRect(destRect, acanvas, srcRect);
  //bmBG1.SaveToFile('W:\sw\kenLCD3\bg.bmp');     //todo
end;

procedure TKenLCD.DrawWithBackground(aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
var
  i, j : integer;
begin
  // resize if required
  if ((hPal = 0) or (awidth <> fbmout.width) or
      (aheight <> fbmOut.height)) then begin
    SetupLCDSize(awidth,aheight,kbm.filename);
  end;

  // copy blank canvas;
  bmBG2.Canvas.Draw(0,0,bmBG1);

  // write on segs
  for i := 0 to kbm.width-1 do begin
    for j := 0 to kbm.height-1 do begin
      if (kbm.data[i,j] <> kbm.backgroundColour) then begin
        if segOn[kbm.data[i,j]] then bmBG2.Canvas.Pixels[i,j] := onColor;
      end;
    end;
  end;

  // write off segs (only if required)
  if offColor <> clNone then begin
    for i := 0 to kbm.width-1 do begin
      for j := 0 to kbm.height-1 do begin
        if (kbm.data[i,j] <> kbm.backgroundColour) then begin
          if not segOn[kbm.data[i,j]] then begin
            bmBG2.Canvas.Pixels[i,j] := offColor;
          end;
        end;
      end;
    end;
  end;

  // draw onto customer canvas
  aCanvas.CopyMode := cmSrcCopy;
  aCanvas.Draw(aleft,atop,bmBG2);
end;

procedure TKenLCD.DrawOnOffWithBackground(onOff:boolean; aleft, atop, awidth, aheight : integer; acanvas : TCanvas);
var
  i, j : integer;
  c : TColor;
begin
  // resize if required
  if ((hPal = 0) or (awidth <> fbmout.width) or
      (aheight <> fbmOut.height)) then begin
    SetupLCDSize(awidth,aheight,kbm.filename);
  end;

  // copy blank canvas;
  bmBG2.Canvas.Draw(0,0,bmBG1);

  if onOff then c := onColor
  else c := offColor;

  // write all segs on or off segs
  if c <> clNone then begin
    // write on segs
    for i := 0 to kbm.width-1 do begin
      for j := 0 to kbm.height-1 do begin
        if (kbm.data[i,j] <> kbm.backgroundColour) then begin
          bmBG2.Canvas.Pixels[i,j] := c;
        end;
      end;
    end;
  end;

  // draw onto customer canvas
  aCanvas.CopyMode := cmSrcCopy;
  aCanvas.Draw(aleft,atop,bmBG2);
end;


procedure TKenLCD.SetupImage(w, h : integer);
var
  i : integer;
  lpPalette : PLogPalette;
begin
  //Allocate the memory used by the palette
  GetMem(lpPalette,sizeof(TLogPalette) + (PalOn * sizeof(TPaletteEntry)));
  try

    //Fill out the palette header
    lpPalette^.palVersion := $300;
    lpPalette^.palNumEntries := 256;

    //Turn range checking off if it is on and remember the range checking state
    {$IFOPT R+}
      {$DEFINE CKRANGE}
      {$R-}
    {$ENDIF}

    //Fill in the palette structure color table
    i := PalOff;
    lpPalette^.PalPalEntry[i].peRed   := offColor and $000000FF;
    lpPalette^.PalPalEntry[i].peGreen := (offColor shr 8) and $000000FF;
    lpPalette^.PalPalEntry[i].peBlue  := (offColor shr 16) and $000000FF;
    lpPalette^.PalPalEntry[i].peflags := PalFlag;
    i := PalOn;
    lpPalette^.PalPalEntry[i].peRed   := onColor and $000000FF;
    lpPalette^.PalPalEntry[i].peGreen := (onColor shr 8) and $000000FF;
    lpPalette^.PalPalEntry[i].peBlue  := (onColor shr 16) and $000000FF;
    lpPalette^.PalPalEntry[i].peflags := PalFlag;

    //Turn range checking back on if it was on when we started
    {$IFDEF CKRANGE}
      {$UNDEF CKRANGE}
      {$R+}
    {$ENDIF}

    hPal := CreatePalette(lpPalette^);
    fbmOut.Palette := hpal;

  finally
    FreeMem(lpPalette, sizeof(TLogPalette) + (PalOn * sizeof(TPaletteEntry)));
  end;
end;

procedure TKenLCD.SetASegment(segnum : Integer; onoff : Boolean);
begin
  if (segnum < kbm.numColours) then
  begin
    segOn[segnum] := onoff;
  end;
end;

procedure TKenLCD.SetSegsOnOff(hex : string);
var
  i : integer;
  b : integer;
begin
  for i := 1 to length (hex) do
  begin
    b := 0;
    if (hex[i] >= '0') and (hex[i] <= '9') then b := ord(hex[i]) - ord('0');
    if (hex[i] >= 'a') and (hex[i] <= 'f') then b := ord(hex[i]) - ord('a') + 10;
    if (hex[i] >= 'A') and (hex[i] <= 'F') then b := ord(hex[i]) - ord('A') + 10;

    SetASegment((i - 1) * 4 + 0, (b and $08) <> 0);
    SetASegment((i - 1) * 4 + 1, (b and $04) <> 0);
    SetASegment((i - 1) * 4 + 2, (b and $02) <> 0);
    SetASegment((i - 1) * 4 + 3, (b and $01) <> 0);
  end;
end;

function TKenLCD.GetSegsOnOff(numBytes : integer) : string;
var
  i:integer;
  b : integer;
begin
  result := '';

  for i := 0 to numBytes-1 do begin
    if (i * 4) >= kbm.numColours then exit;

    b := 0;
    if segOn[(i*4)+0] then b := b + $08;
    if segOn[(i*4)+1] then b := b + $04;
    if segOn[(i*4)+2] then b := b + $02;
    if segOn[(i*4)+3] then b := b + $01;

    if (b <= 9) then result := result + chr(b+ord('0'))
    else result := result + chr(b-10+ord('A'));
  end;
end;

function TKenLCD.GetNumSegments : Integer;
begin
  Result := kbm.numColours;
end;

end.
