unit loginfo;

interface

uses Graphics, Series, TEEngine;

const
  LOG_TYPE_IDX_NONE = 0;
  LOG_TYPE_IDX_MOTION = 1;
  LOG_TYPE_IDX_ZERO = 2;
  LOG_TYPE_IDX_OUTPUT1 = 3;
  LOG_TYPE_IDX_OUTPUT2 = 4;
  LOG_TYPE_IDX_OUTPUT3 = 5;
  LOG_TYPE_IDX_OUTPUT4 = 6;
  LOG_TYPE_IDX_OUTPUT5 = 7;
  LOG_TYPE_IDX_OUTPUT6 = 8;
  LOG_TYPE_IDX_OUTPUT7 = 9;
  LOG_TYPE_IDX_OUTPUT8 = 10;
  NUM_LOG_TYPE_IDX = 11;

  COF11_MASK_OVERLOAD = $01;
  COF11_MASK_STANDSTILL = $02;
  COF11_MASK_GROSS = $04;
  COF11_MASK_RANGE2 = $08;
  COF11_MASK_IO1 = $10;
  COF11_MASK_IO2 = $20;
  COF11_MASK_IO3 = $40;
  COF11_MASK_IO4 = $80;
  COF11_MASK_COZ = $100;

  COF12_MASK_IO1 = $01;
  COF12_MASK_IO2 = $02;
  COF12_MASK_IO3 = $04;
  COF12_MASK_IO4 = $08;
  COF12_MASK_IO5 = $10;
  COF12_MASK_IO6 = $20;
  COF12_MASK_IO7 = $40;
  COF12_MASK_IO8 = $80;

  LogTypeNames : Array[0..NUM_LOG_TYPE_IDX - 1] of String =
  (
    'None', 'Motion', 'Zero',
    'I/O 1', 'I/O 2', 'I/O 3', 'I/O 4',
    'I/O 5', 'I/O 6', 'I/O 7', 'I/O 8'
  );

  LogTypeToCof11Mask : Array[0..NUM_LOG_TYPE_IDX - 1] of Integer =
  (
    $00, COF11_MASK_STANDSTILL, COF11_MASK_COZ,
    COF11_MASK_IO1, COF11_MASK_IO2, COF11_MASK_IO3, COF11_MASK_IO4,
    $00, $00, $00, $00
  );

  LogTypeToCof12Mask : Array[0..NUM_LOG_TYPE_IDX - 1] of Integer =
  (
    $00, $00, $00,
    COF12_MASK_IO1, COF12_MASK_IO2, COF12_MASK_IO3, COF12_MASK_IO4,
    COF12_MASK_IO5, COF12_MASK_IO6, COF12_MASK_IO7, COF12_MASK_IO8
  );

  LogTypeIsIO : Array[0..NUM_LOG_TYPE_IDX - 1] of Boolean =
  (
    False, False, False,
    True, True, True, True,
    True, True, True, True
  );

  LogTypeRequiresInversion : Array[0..NUM_LOG_TYPE_IDX - 1] of Boolean =
  (
    False, True, False,
    False, False, False, False,
    False, False, False, False
  );


type

  TLoggingInfo = Class(TObject)

  private
    FLogType: Integer;
    FSeries: TLineSeries;
    FChartYOffset: Double;

    function CalculateDigitalYValue(state : Boolean) : Double;
    function GetSeriesYValue(idx : Longint) : Double;
  public
    constructor Create(ASeries: TLineSeries; AChartYOffset : Double); overload;
    destructor Destroy; override;
    procedure SetLogType(ALogType : Integer);
    function GetLogType : Integer;
    procedure AddSeriesDataPoint(time : Double; state : Boolean); overload;
    procedure AddSeriesDataPoint(time : Double; usecof12 : Boolean; status : Integer; cof12io : Integer); overload;
    function GetSeriesYDigitalValue(idx : Longint) : Boolean;
    procedure SetupSeries;
  end;

procedure SetupSeries(Series : TChartSeries; Enabled : Boolean; Name : String);

implementation

procedure SetupSeries(Series : TChartSeries; Enabled : Boolean; Name : String);
begin
  Series.ShowInLegend := False;
  Series.Title := Name;
  Series.ShowInLegend := Enabled;
end;

constructor TLoggingInfo.Create(ASeries: TLineSeries; AChartYOffset : Double);
begin
  inherited Create();
  FLogType := LOG_TYPE_IDX_NONE;
  FSeries := ASeries;
  FChartYOffset := AChartYOffset;
end;

destructor TLoggingInfo.Destroy;
begin
  FSeries := nil;
  inherited;
end;

procedure TLoggingInfo.SetLogType(ALogType : Integer);
begin
  if (ALogType >= 0) and (ALogType < NUM_LOG_TYPE_IDX) then
    FLogType := ALogType;
  SetupSeries();
end;

function TLoggingInfo.GetLogType : Integer;
begin
  Result := FLogType;
end;

function TLoggingInfo.CalculateDigitalYValue(state : Boolean) : Double;
begin
  Result := FChartYOffset;
  if (state) then
    Result := Result + 1;
end;

procedure TLoggingInfo.AddSeriesDataPoint(time : Double; state : Boolean);
begin
  if (FLogType <> LOG_TYPE_IDX_NONE) then
  begin
    FSeries.AddXY(time, CalculateDigitalYValue(state), '', clTeeColor);
  end;
end;

procedure TLoggingInfo.AddSeriesDataPoint(time : Double; usecof12 : Boolean; status : Integer; cof12io : Integer);
var state : Boolean;
begin
  if (FLogType <> LOG_TYPE_IDX_NONE) then
  begin
    if (usecof12) then
    begin
      if (LogTypeIsIO[FLogType]) then
      begin
        state := ((LogTypeToCof12Mask[FLogType] and cof12io) <> 0);
      end
      else
      begin
        state := ((LogTypeToCof11Mask[FLogType] and status) <> 0);
      end;
    end
    else
    begin
      // Only IOs1..4 available
      state := ((LogTypeToCof11Mask[FLogType] and status) <> 0);
    end;

    if (LogTypeRequiresInversion[FLogType]) then
      state := not state;

    AddSeriesDataPoint(time, state);
  end;
end;

procedure TLoggingInfo.SetupSeries;
begin
  loginfo.SetupSeries(FSeries, (FLogType <> LOG_TYPE_IDX_NONE), LogTypeNames[FLogType]);
  FSeries.Stairs := True;
end;

function TLoggingInfo.GetSeriesYValue(idx : Longint) : Double;
begin
  Result := FSeries.YValues[idx];
end;

function TLoggingInfo.GetSeriesYDigitalValue(idx : Longint) : Boolean;
var value : Double;
begin
  value := GetSeriesYValue(idx) / FSeries.YValues.Multiplier;
  value := value - FChartYOffset;
  Result := (value > 0.5);
end;

end.
